@@ -1,16 +1,14 @@
 // SPDX-FileCopyrightText: 2022 Kevin Matthes
 // SPDX-FileCopyrightText: 2022 Shun Sakai
+// SPDX-FileCopyrightText: 2023 zSchoen
 //
 // SPDX-License-Identifier: Apache-2.0 OR MIT
 
-//! The system exit code constants as defined by
-//! [`<sysexits.h>`][sysexits-man-url].
+//! The system exit code constants as defined by [`<sysexits.h>`].
 //!
-//! [sysexits-man-url]: https://man.openbsd.org/sysexits
+//! [`<sysexits.h>`]: https://man.openbsd.org/sysexits
 
 use core::fmt;
-#[cfg(feature = "std")]
-use std::process::Termination;
 
 /// A [`Result`](std::result::Result) type based on [`ExitCode`].
 ///
@@ -20,9 +18,9 @@ use std::process::Termination;
 pub type Result<T> = std::result::Result<T, ExitCode>;
 
 /// `ExitCode` is a type that represents the system exit code constants as
-/// defined by [`<sysexits.h>`][sysexits-man-url].
+/// defined by [`<sysexits.h>`].
 ///
-/// [sysexits-man-url]: https://man.openbsd.org/sysexits
+/// [`<sysexits.h>`]: https://man.openbsd.org/sysexits
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum ExitCode {
     /// The successful exit.
@@ -124,8 +122,8 @@ pub enum ExitCode {
 
     /// An operating system error has been detected. This is intended to be used
     /// for such things as "cannot fork", or "cannot create pipe". It includes
-    /// things like [`getuid(2)`][getuid-2-man-url] returning a user that does
-    /// not exist in the passwd file.
+    /// things like [`getuid(2)`] returning a user that does not exist in the
+    /// passwd file.
     ///
     /// # Examples
     ///
@@ -135,7 +133,7 @@ pub enum ExitCode {
     /// assert_eq!(ExitCode::OsErr as u8, 71);
     /// ```
     ///
-    /// [getuid-2-man-url]: https://man.openbsd.org/getuid.2
+    /// [`getuid(2)`]: https://man.openbsd.org/getuid.2
     OsErr,
 
     /// Some system file (e.g., `/etc/passwd`, `/var/run/utmp`) does not exist,
@@ -260,8 +258,7 @@ impl ExitCode {
 
     /// Terminates the current process with the exit code defined by `ExitCode`.
     ///
-    /// This method is equivalent to [`std::process::exit`] with a restricted
-    /// exit code.
+    /// Equivalent to [`std::process::exit`] with a restricted exit code.
     ///
     /// # Examples
     ///
@@ -328,17 +325,26 @@ macro_rules! impl_from_exit_code_for_integer {
         );
     };
 }
+impl_from_exit_code_for_integer!(i8);
+impl_from_exit_code_for_integer!(i16);
 impl_from_exit_code_for_integer!(i32);
+impl_from_exit_code_for_integer!(i64);
+impl_from_exit_code_for_integer!(i128);
+impl_from_exit_code_for_integer!(isize);
 impl_from_exit_code_for_integer!(u8);
+impl_from_exit_code_for_integer!(u16);
 impl_from_exit_code_for_integer!(u32);
+impl_from_exit_code_for_integer!(u64);
+impl_from_exit_code_for_integer!(u128);
+impl_from_exit_code_for_integer!(usize);
 
 #[cfg(feature = "std")]
 impl From<ExitCode> for std::process::ExitCode {
     /// Converts an `sysexits::ExitCode` into an [`std::process::ExitCode`].
-    ///
-    /// This method is equivalent to [`ExitCode::report`].
     #[inline]
     fn from(code: ExitCode) -> Self {
+        use std::process::Termination;
+
         code.report()
     }
 }
@@ -420,9 +426,15 @@ impl From<std::io::ErrorKind> for ExitCode {
             | ErrorKind::BrokenPipe
             | ErrorKind::TimedOut
             | ErrorKind::Interrupted => Self::TempFail,
+            #[cfg(feature = "extended_io_error")]
+            ErrorKind::HostUnreachable | ErrorKind::NetworkUnreachable => Self::NoHost,
             ErrorKind::AddrInUse | ErrorKind::AddrNotAvailable => Self::Unavailable,
+            #[cfg(feature = "extended_io_error")]
+            ErrorKind::NetworkDown => Self::Unavailable,
             ErrorKind::AlreadyExists => Self::CantCreat,
             ErrorKind::WouldBlock | ErrorKind::Unsupported => Self::Protocol,
+            #[cfg(feature = "extended_io_error")]
+            ErrorKind::ReadOnlyFilesystem => Self::CantCreat,
             ErrorKind::InvalidInput | ErrorKind::InvalidData => Self::DataErr,
             ErrorKind::WriteZero | ErrorKind::UnexpectedEof => Self::Software,
             _ => Self::IoErr,
@@ -438,7 +450,7 @@ impl TryFrom<std::process::ExitStatus> for ExitCode {
     ///
     /// # Errors
     ///
-    /// This method returns [`Err`] in the following situations:
+    /// Returns [`Err`] if any of the following are true:
     ///
     /// - The exit code is not `0` or `64..=78`.
     /// - The exit code is unknown (e.g., the process was terminated by a
@@ -471,10 +483,10 @@ impl TryFrom<std::process::ExitStatus> for ExitCode {
 impl std::error::Error for ExitCode {}
 
 #[cfg(feature = "std")]
-impl Termination for ExitCode {
+impl std::process::Termination for ExitCode {
     #[inline]
     fn report(self) -> std::process::ExitCode {
-        std::process::ExitCode::from(u8::from(self))
+        u8::from(self).into()
     }
 }
 
@@ -865,7 +877,7 @@ mod tests {
         assert!(ExitCode::Config.is_failure());
     }
 
-    macro_rules! from_exit_code_to_integer {
+    macro_rules! test_from_exit_code_to_integer {
         ($T:ty, $name:ident) => {
             #[test]
             fn $name() {
@@ -888,9 +900,18 @@ mod tests {
             }
         };
     }
-    from_exit_code_to_integer!(i32, from_exit_code_to_i32);
-    from_exit_code_to_integer!(u8, from_exit_code_to_u8);
-    from_exit_code_to_integer!(u32, from_exit_code_to_u32);
+    test_from_exit_code_to_integer!(i8, from_exit_code_to_i8);
+    test_from_exit_code_to_integer!(i16, from_exit_code_to_i16);
+    test_from_exit_code_to_integer!(i32, from_exit_code_to_i32);
+    test_from_exit_code_to_integer!(i64, from_exit_code_to_i64);
+    test_from_exit_code_to_integer!(i128, from_exit_code_to_i128);
+    test_from_exit_code_to_integer!(isize, from_exit_code_to_isize);
+    test_from_exit_code_to_integer!(u8, from_exit_code_to_u8);
+    test_from_exit_code_to_integer!(u16, from_exit_code_to_u16);
+    test_from_exit_code_to_integer!(u32, from_exit_code_to_u32);
+    test_from_exit_code_to_integer!(u64, from_exit_code_to_u64);
+    test_from_exit_code_to_integer!(u128, from_exit_code_to_u128);
+    test_from_exit_code_to_integer!(usize, from_exit_code_to_usize);
 
     #[cfg(feature = "std")]
     #[test]
@@ -1104,6 +1125,7 @@ mod tests {
     }
 
     #[cfg(feature = "std")]
+    #[allow(clippy::too_many_lines)]
     #[test]
     fn from_io_error_to_exit_code() {
         use std::io::{Error, ErrorKind};
@@ -1124,6 +1146,16 @@ mod tests {
             ExitCode::from(Error::from(ErrorKind::ConnectionReset)),
             ExitCode::TempFail
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::HostUnreachable)),
+            ExitCode::NoHost
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::NetworkUnreachable)),
+            ExitCode::NoHost
+        );
         assert_eq!(
             ExitCode::from(Error::from(ErrorKind::ConnectionAborted)),
             ExitCode::TempFail
@@ -1140,6 +1172,11 @@ mod tests {
             ExitCode::from(Error::from(ErrorKind::AddrNotAvailable)),
             ExitCode::Unavailable
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::NetworkDown)),
+            ExitCode::Unavailable
+        );
         assert_eq!(
             ExitCode::from(Error::from(ErrorKind::BrokenPipe)),
             ExitCode::TempFail
@@ -1152,6 +1189,36 @@ mod tests {
             ExitCode::from(Error::from(ErrorKind::WouldBlock)),
             ExitCode::Protocol
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::NotADirectory)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::IsADirectory)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::DirectoryNotEmpty)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::ReadOnlyFilesystem)),
+            ExitCode::CantCreat
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::FilesystemLoop)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::StaleNetworkFileHandle)),
+            ExitCode::IoErr
+        );
         assert_eq!(
             ExitCode::from(Error::from(ErrorKind::InvalidInput)),
             ExitCode::DataErr
@@ -1168,6 +1235,61 @@ mod tests {
             ExitCode::from(Error::from(ErrorKind::WriteZero)),
             ExitCode::Software
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::StorageFull)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::NotSeekable)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::FilesystemQuotaExceeded)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::FileTooLarge)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::ResourceBusy)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::ExecutableFileBusy)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::Deadlock)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::CrossesDevices)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::TooManyLinks)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::InvalidFilename)),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(Error::from(ErrorKind::ArgumentListTooLong)),
+            ExitCode::IoErr
+        );
         assert_eq!(
             ExitCode::from(Error::from(ErrorKind::Interrupted)),
             ExitCode::TempFail
@@ -1191,7 +1313,7 @@ mod tests {
     }
 
     #[cfg(feature = "std")]
-    #[allow(clippy::cognitive_complexity)]
+    #[allow(clippy::cognitive_complexity, clippy::too_many_lines)]
     #[test]
     fn from_io_error_kind_to_exit_code() {
         use std::io;
@@ -1209,6 +1331,16 @@ mod tests {
             ExitCode::from(io::ErrorKind::ConnectionReset),
             ExitCode::TempFail
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::HostUnreachable),
+            ExitCode::NoHost
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::NetworkUnreachable),
+            ExitCode::NoHost
+        );
         assert_eq!(
             ExitCode::from(io::ErrorKind::ConnectionAborted),
             ExitCode::TempFail
@@ -1225,6 +1357,11 @@ mod tests {
             ExitCode::from(io::ErrorKind::AddrNotAvailable),
             ExitCode::Unavailable
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::NetworkDown),
+            ExitCode::Unavailable
+        );
         assert_eq!(
             ExitCode::from(io::ErrorKind::BrokenPipe),
             ExitCode::TempFail
@@ -1237,6 +1374,33 @@ mod tests {
             ExitCode::from(io::ErrorKind::WouldBlock),
             ExitCode::Protocol
         );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::NotADirectory),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::IsADirectory), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::DirectoryNotEmpty),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::ReadOnlyFilesystem),
+            ExitCode::CantCreat
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::FilesystemLoop),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::StaleNetworkFileHandle),
+            ExitCode::IoErr
+        );
         assert_eq!(
             ExitCode::from(io::ErrorKind::InvalidInput),
             ExitCode::DataErr
@@ -1245,8 +1409,47 @@ mod tests {
             ExitCode::from(io::ErrorKind::InvalidData),
             ExitCode::DataErr
         );
+        #[cfg(feature = "extended_io_error")]
         assert_eq!(ExitCode::from(io::ErrorKind::TimedOut), ExitCode::TempFail);
+        #[cfg(feature = "extended_io_error")]
         assert_eq!(ExitCode::from(io::ErrorKind::WriteZero), ExitCode::Software);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::StorageFull), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::NotSeekable), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::FilesystemQuotaExceeded),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::FileTooLarge), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::ResourceBusy), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::ExecutableFileBusy),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::Deadlock), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::CrossesDevices),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(ExitCode::from(io::ErrorKind::TooManyLinks), ExitCode::IoErr);
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::InvalidFilename),
+            ExitCode::IoErr
+        );
+        #[cfg(feature = "extended_io_error")]
+        assert_eq!(
+            ExitCode::from(io::ErrorKind::ArgumentListTooLong),
+            ExitCode::IoErr
+        );
         assert_eq!(
             ExitCode::from(io::ErrorKind::Interrupted),
             ExitCode::TempFail
@@ -1404,6 +1607,8 @@ mod tests {
     #[cfg(feature = "std")]
     #[test]
     fn report_exit_code() {
+        use std::process::Termination;
+
         assert_eq!(
             format!("{:?}", ExitCode::Ok.report()),
             format!("{:?}", std::process::ExitCode::from(0))
