@@ -1,10 +1,11 @@
 // SPDX-FileCopyrightText: 2022 Shun Sakai
 // SPDX-FileCopyrightText: 2023 Kevin Matthes
+// SPDX-FileCopyrightText: 2023 zSchoen
 //
 // SPDX-License-Identifier: Apache-2.0 OR MIT
 
 //! The `sysexits` crate provides the system exit code constants as defined by
-//! [`<sysexits.h>`][sysexits-man-url].
+//! [`<sysexits.h>`].
 //!
 //! The [`ExitCode`] type implements the
 //! [`Termination`](std::process::Termination) trait, so this can be returned
@@ -12,23 +13,24 @@
 //!
 //! # Examples
 //!
-//! ## Returns the exit code as defined by <sysexits.h>
-//!
-//! If you only use the exit code as defined by `<sysexits.h>`, you can return
-//! this from the `main` function.
-//!
 //! ```
 //! # #[cfg(feature = "std")]
-//! fn main() -> sysexits::ExitCode {
+//! use std::str;
+//!
+//! # #[cfg(feature = "std")]
+//! use sysexits::ExitCode;
+//!
+//! # #[cfg(feature = "std")]
+//! fn main() -> ExitCode {
 //!     let bytes = [0xf0, 0x9f, 0x92, 0x96];
-//!     match std::str::from_utf8(&bytes) {
+//!     match str::from_utf8(&bytes) {
 //!         Ok(string) => {
 //!             println!("{string}");
-//!             sysexits::ExitCode::Ok
+//!             ExitCode::Ok
 //!         }
 //!         Err(err) => {
 //!             eprintln!("{err}");
-//!             sysexits::ExitCode::DataErr
+//!             ExitCode::DataErr
 //!         }
 //!     }
 //! }
@@ -37,38 +39,10 @@
 //! # fn main() {}
 //! ```
 //!
-//! ## Combine with other exit codes
-//!
-//! [`ExitCode`] can be converted to [`std::process::ExitCode`] by the [`From`]
-//! trait, so you can combine it with your own exit codes or
-//! [`std::process::ExitCode`].
-//!
-//! ```
-//! # #[cfg(feature = "std")]
-//! use std::io::Read;
-//!
-//! # #[cfg(feature = "std")]
-//! fn main() -> std::process::ExitCode {
-//!     let mut buf = String::new();
-//!     if let Err(err) = std::io::stdin().read_to_string(&mut buf) {
-//!         eprintln!("{err}");
-//!         sysexits::ExitCode::try_from(err.kind()).map_or(
-//!             std::process::ExitCode::FAILURE,
-//!             std::process::ExitCode::from,
-//!         )
-//!     } else {
-//!         print!("{buf}");
-//!         std::process::ExitCode::SUCCESS
-//!     }
-//! }
-//! #
-//! # #[cfg(not(feature = "std"))]
-//! # fn main() {}
-//! ```
-//!
-//! [sysexits-man-url]: https://man.openbsd.org/sysexits
+//! [`<sysexits.h>`]: https://man.openbsd.org/sysexits
 
-#![doc(html_root_url = "https://docs.rs/sysexits/0.7.2/")]
+#![cfg_attr(feature = "extended_io_error", feature(io_error_more))]
+#![doc(html_root_url = "https://docs.rs/sysexits/0.7.11/")]
 #![no_std]
 #![cfg_attr(doc_cfg, feature(doc_auto_cfg, doc_cfg))]
 // Lint levels of rustc.
@@ -88,8 +62,6 @@ extern crate std;
 mod error;
 mod exit_code;
 
-#[cfg(feature = "std")]
-pub use crate::error::TryFromExitStatusError;
 pub use crate::exit_code::ExitCode;
 #[cfg(feature = "std")]
-pub use crate::exit_code::Result;
+pub use crate::{error::TryFromExitStatusError, exit_code::Result};
